/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2014 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.base.ui.annotation;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;

import org.fortiss.tooling.base.model.element.IAnnotatedSpecification;
import org.fortiss.tooling.base.model.element.IModelElement;

/**
 * Class for an annotation entry in the annotation view table
 * 
 * @author eder
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating RED Hash:
 */
final class AnnotationEntry {

	/** Value not availabe */
	public static final String NOVAL = "-";

	/** Model Element */
	private IModelElement modelElement;

	/** Hash map {@link IAnnotatedSpecification} -> {@link IAnnotationValueProvider} */
	HashMap<Class<? extends IAnnotatedSpecification>, IAnnotationValueProvider<IAnnotatedSpecification>> providerSpecMapping =
			new HashMap<Class<? extends IAnnotatedSpecification>, IAnnotationValueProvider<IAnnotatedSpecification>>();

	/** List of specifications of {@link #modelElement} */
	private List<IAnnotatedSpecification> specificationsList =
			new ArrayList<IAnnotatedSpecification>();

	/** Constructor */
	public AnnotationEntry(IModelElement modelElement) {
		this.modelElement = modelElement;

	}

	/** adds a new touple {@link IAnnotationValueProvider} {@link IAnnotatedSpecification} */
	public void addNewSpecification(IAnnotationValueProvider<IAnnotatedSpecification> provider,
			IAnnotatedSpecification spec) {
		providerSpecMapping.put(spec.getClass(), provider);
		specificationsList.add(spec);
	}

	/** Returns the string representation of the annotation value */
	public String getSpecificationValue(Class<? extends IAnnotatedSpecification> clazz) {
		for(IAnnotatedSpecification s : specificationsList) {
			if(clazz.isInstance(s)) {
				return providerSpecMapping.get(clazz).getValue(s);
			}
		}
		return NOVAL;
	}

	/** Returns the possible values of the the given specification clazz */
	public List<String> getFixedSpecificationValues(Class<? extends IAnnotatedSpecification> clazz) {
		for(IAnnotatedSpecification s : specificationsList) {
			if(clazz.isInstance(s)) {
				return providerSpecMapping.get(clazz).getFixedValues();
			}
		}
		return Collections.emptyList();
	}

	/** adds a new value to the given specification clazz and overrides the old one */
	public void setSpecificationValue(String value, Class<? extends IAnnotatedSpecification> clazz)
			throws Exception {
		for(IAnnotatedSpecification s : specificationsList) {
			if(clazz.isInstance(s)) {
				providerSpecMapping.get(clazz).setValue(value, s);
				return;
			}
		}

		throw new Exception("Could not find a AnnotationValueProvider for " + clazz.toString());
	}

	/** Returns modelElement. */
	public IModelElement getModelElement() {
		return modelElement;
	}

	/** Returns the annotation name */
	public String getAnnotationName(Class<? extends IAnnotatedSpecification> clazz) {
		for(IAnnotatedSpecification s : specificationsList) {
			if(clazz.isInstance(s)) {
				return providerSpecMapping.get(clazz).getAnnotationName();
			}
		}
		return "Provider not found";
	}

	/** Returns specificationsList. */
	public List<IAnnotatedSpecification> getSpecificationsList() {
		return specificationsList;
	}
}
